@@ -1,8 +1,10 @@
+from typing import Tuple
+
 from .IAbortable import IAbortable
 
 
 class IAutoFocus(IAbortable):
-    def auto_focus(self, count: int, step: float, exposure_time: int, *args, **kwargs) -> (float, float):
+    def auto_focus(self, count: int, step: float, exposure_time: int, *args, **kwargs) -> Tuple[float, float]:
         """Perform an auto-focus series.
 
         This method performs an auto-focus series with "count" images on each side of the initial guess and the given
