@@ -1,10 +1,12 @@
+from typing import Tuple
+
 from .ITemperatures import ITemperatures
 
 
 class ICooling(ITemperatures):
     """Interface for all devices that allow for some kind of cooling."""
 
-    def get_cooling_status(self, *args, **kwargs) -> (bool,  float, float):
+    def get_cooling_status(self, *args, **kwargs) -> Tuple[bool, float, float]:
         """Returns the current status for the cooling.
 
         Returns:
