@@ -9,8 +9,9 @@ from .IAutonomous import IAutonomous
 from .ICalibrate import ICalibrate
 from .ICamera import ICamera
 from .ICameraBinning import ICameraBinning
+from .ICameraExposureTime import ICameraExposureTime
 from .ICameraWindow import ICameraWindow
-from .IConfigProvider import IConfigProvider
+from .IConfig import IConfig
 from .ICooling import ICooling
 from .ICoordinatesQuery import ICoordinatesQuery
 from .IDome import IDome
@@ -19,6 +20,8 @@ from .IFitsHeaderProvider import IFitsHeaderProvider
 from .IFlatField import IFlatField
 from .IFocusModel import IFocusModel
 from .IFocuser import IFocuser
+from .IImageType import IImageType
+from .IModule import IModule
 from .IMotion import IMotion
 from .IPipeline import IPipeline
 from .IRaDec import IRaDec
@@ -28,7 +31,6 @@ from .IRoof import IRoof
 from .IRotation import IRotation
 from .IRunnable import IRunnable
 from .IScriptRunner import IScriptRunner
-from .ISettings import ISettings
 from .IStoppable import IStoppable
 from .ISyncTarget import ISyncTarget
 from .ITelescope import ITelescope
