@@ -3,9 +3,9 @@ import threading
 import typing
 
 from pyobs.interfaces import IRunnable, IFlatField
-from pyobs import PyObsModule
+from pyobs import Module
 from pyobs.modules import timeout
-from pyobs.object import create_object
+from pyobs.object import get_object
 from pyobs.utils.skyflats.priorities.base import SkyflatPriorities
 from pyobs.utils.skyflats.scheduler import Scheduler, SchedulerItem
 from pyobs.utils.threads import Future
@@ -14,7 +14,7 @@ from pyobs.utils.time import Time
 log = logging.getLogger(__name__)
 
 
-class FlatFieldScheduler(PyObsModule, IRunnable):
+class FlatFieldScheduler(Module, IRunnable):
     """Run the flat-field scheduler."""
 
     def __init__(self, flatfield: typing.Union[str, IFlatField], functions: typing.Dict[str, str],
@@ -32,7 +32,7 @@ class FlatFieldScheduler(PyObsModule, IRunnable):
             filter_change: Time required for filter change [s]
             count: Number of flats to take per filter/binning
         """
-        PyObsModule.__init__(self, *args, **kwargs)
+        Module.__init__(self, *args, **kwargs)
 
         # store
         self._flatfield = flatfield
@@ -42,16 +42,16 @@ class FlatFieldScheduler(PyObsModule, IRunnable):
         self._abort = threading.Event()
 
         # priorities
-        priorities = create_object(priorities, SkyflatPriorities)
+        prio = get_object(priorities, SkyflatPriorities)
 
         # create scheduler
-        self._scheduler = Scheduler(functions, priorities, self.observer, min_exptime=min_exptime,
+        self._scheduler = Scheduler(functions, prio, self.observer, min_exptime=min_exptime,
                                     max_exptime=max_exptime, timespan=timespan, filter_change=filter_change,
                                     count=count)
 
     def open(self):
         """Open module"""
-        PyObsModule.open(self)
+        Module.open(self)
 
         # check flat field
         try:
@@ -59,7 +59,7 @@ class FlatFieldScheduler(PyObsModule, IRunnable):
         except ValueError:
             log.warning('Flatfield module does not exist or is not of correct type at the moment.')
 
-    @timeout(7200000)
+    @timeout(7200)
     def run(self, *args, **kwargs):
         """Perform flat-fielding"""
         log.info('Performing flat fielding...')
@@ -98,7 +98,7 @@ class FlatFieldScheduler(PyObsModule, IRunnable):
         # finished
         log.info('Finished.')
 
-    @timeout(20000)
+    @timeout(20)
     def abort(self, *args, **kwargs):
         """Abort current actions."""
         self._abort.set()
