@@ -1,20 +1,21 @@
 import logging
-from typing import Union
+from typing import Union, Tuple
 import threading
 import numpy as np
 
 from pyobs.comm import RemoteException
-from pyobs.interfaces import IFocuser, ICamera, IAutoFocus, IFilters
+from pyobs.interfaces import IFocuser, ICamera, IAutoFocus, IFilters, ICameraExposureTime, IImageType
 from pyobs.events import FocusFoundEvent
-from pyobs import PyObsModule, get_object
+from pyobs import Module, get_object
 from pyobs.mixins import CameraSettingsMixin
 from pyobs.modules import timeout
+from pyobs.utils.enums import ImageType
 from pyobs.utils.focusseries import FocusSeries
 
 log = logging.getLogger(__name__)
 
 
-class AutoFocusSeries(PyObsModule, CameraSettingsMixin, IAutoFocus):
+class AutoFocusSeries(Module, CameraSettingsMixin, IAutoFocus):
     """Module for auto-focusing a telescope."""
 
     def __init__(self, focuser: Union[str, IFocuser], camera: Union[str, ICamera], filters: Union[str, IFilters],
@@ -27,7 +28,7 @@ class AutoFocusSeries(PyObsModule, CameraSettingsMixin, IAutoFocus):
             filters: Name of IFilters, if any.
             offset: If True, offsets are used instead of absolute focus values.
         """
-        PyObsModule.__init__(self, *args, **kwargs)
+        Module.__init__(self, *args, **kwargs)
 
         # store focuser and camera
         self._focuser = focuser
@@ -39,16 +40,12 @@ class AutoFocusSeries(PyObsModule, CameraSettingsMixin, IAutoFocus):
         # create focus series
         self._series: FocusSeries = get_object(series, FocusSeries)
 
-        # storage for data
-        self._data_lock = threading.RLock()
-        self._data = []
-
         # init camera settings mixin
         CameraSettingsMixin.__init__(self, *args, filters=filters, **kwargs)
 
     def open(self):
         """Open module"""
-        PyObsModule.open(self)
+        Module.open(self)
 
         # register event
         self.comm.register_event(FocusFoundEvent)
@@ -63,8 +60,8 @@ class AutoFocusSeries(PyObsModule, CameraSettingsMixin, IAutoFocus):
     def close(self):
         """Close module."""
 
-    @timeout(600000)
-    def auto_focus(self, count: int, step: float, exposure_time: int, *args, **kwargs) -> (float, float):
+    @timeout(600)
+    def auto_focus(self, count: int, step: float, exposure_time: int, *args, **kwargs) -> Tuple[float, float]:
         """Perform an auto-focus series.
 
         This method performs an auto-focus series with "count" images on each side of the initial guess and the given
@@ -77,10 +74,9 @@ class AutoFocusSeries(PyObsModule, CameraSettingsMixin, IAutoFocus):
             exposure_time: Exposure time for images.
 
         Returns:
-            Tuple of obtained best focus value and its uncertainty.
+            Tuple of obtained best focus value and its uncertainty. Or Nones, if focus series failed.
 
         Raises:
-            ValueError: If focus could not be obtained.
             FileNotFoundException: If image could not be downloaded.
         """
         log.info('Performing auto-focus...')
@@ -102,7 +98,7 @@ class AutoFocusSeries(PyObsModule, CameraSettingsMixin, IAutoFocus):
             filter_wheel: IFilters = self.proxy(self._filters, IFilters)
             filter_name = filter_wheel.get_filter().wait()
         except ValueError:
-            log.warning('Either camera or focuser do not exist or are not of correct type at the moment.')
+            log.warning('Filter module is not of type IFilters. Could not get filter.')
 
         # get focus as first guess
         try:
@@ -142,15 +138,18 @@ class AutoFocusSeries(PyObsModule, CameraSettingsMixin, IAutoFocus):
             if self._abort.is_set():
                 raise InterruptedError()
             try:
-                filename = camera.expose(exposure_time=exposure_time, image_type=ICamera.ImageType.FOCUS,
-                                         count=1).wait()[0]
+                if isinstance(camera, ICameraExposureTime):
+                    camera.set_exposure_time(exposure_time)
+                if isinstance(camera, IImageType):
+                    camera.set_image_type(ImageType.FOCUS)
+                filename = camera.expose().wait()
             except RemoteException:
                 log.error('Could not take image.')
                 continue
 
             # download image
             log.info('Downloading image...')
-            image = self.vfs.download_image(filename)
+            image = self.vfs.read_image(filename)
 
             # analyse
             log.info('Analysing picture...')
@@ -166,9 +165,20 @@ class AutoFocusSeries(PyObsModule, CameraSettingsMixin, IAutoFocus):
             raise InterruptedError()
         focus = self._series.fit_focus()
 
-        # check
+        # did focus series fail?
         if focus is None or focus[0] is None or np.isnan(focus[0]):
-            raise ValueError('Could not fit focus.')
+            log.warning('Focus series failed.')
+
+            # reset to initial values
+            if self._offset:
+                log.info('Resetting focus offset to initial guess of %.3f mm.', guess)
+                focuser.set_focus_offset(focus[0]).wait()
+            else:
+                log.info('Resetting focus to initial guess of %.3f mm.', guess)
+                focuser.set_focus(focus[0]).wait()
+
+            # raise error
+            raise ValueError('Could not find best focus.')
 
         # "absolute" will be the absolute focus value, i.e. focus+offset
         absolute = None
@@ -199,7 +209,7 @@ class AutoFocusSeries(PyObsModule, CameraSettingsMixin, IAutoFocus):
         """
         return {}
 
-    @timeout(20000)
+    @timeout(20)
     def abort(self, *args, **kwargs):
         """Abort current actions."""
         self._abort.set()
