@@ -6,18 +6,18 @@ import pandas as pd
 from astropy.coordinates import SkyCoord
 import astropy.units as u
 
-from pyobs import PyObsModule
+from pyobs import Module
 from pyobs.interfaces import IAcquisition, IAutonomous
 from pyobs.utils.time import Time
 
 log = logging.getLogger(__name__)
 
 
-class PointingSeries(PyObsModule, IAutonomous):
+class PointingSeries(Module, IAutonomous):
     """Module for running pointing series."""
 
     def __init__(self, min_alt: int = 30, max_alt: int = 85, num_alt: int = 8, num_az: int = 24, finish: int = 90,
-                 exp_time: int = 1000, acquisition: str = 'acquisition', *args, **kwargs):
+                 exp_time: float = 1., acquisition: str = 'acquisition', *args, **kwargs):
         """Initialize a new auto focus system.
 
         Args:
@@ -26,10 +26,10 @@ class PointingSeries(PyObsModule, IAutonomous):
             num_alt: Number of altitude points to create on grid.
             num_az: Number of azimuth points to create on grid.
             finish: When this number in percent of points have been finished, terminate mastermind.
-            exp_time: Exposure time in ms.
+            exp_time: Exposure time in secs.
             acquisition: IAcquisition unit to use.
         """
-        PyObsModule.__init__(self, *args, **kwargs)
+        Module.__init__(self, *args, **kwargs)
 
         # store
         self._min_alt = min_alt
