@@ -1,12 +1,12 @@
 import logging
 
-from pyobs import PyObsModule
+from pyobs import Module
 
 
 log = logging.getLogger(__name__)
 
 
-class StandAlone(PyObsModule):
+class StandAlone(Module):
     """Example module that only logs the given message forever in the given interval."""
 
     def __init__(self, message: str = 'Hello world', interval: int = 10, *args, **kwargs):
@@ -16,16 +16,16 @@ class StandAlone(PyObsModule):
             message: Message to log in the given interval.
             interval: Interval between messages.
         """
-        PyObsModule.__init__(self, *args, **kwargs)
+        Module.__init__(self, *args, **kwargs)
 
         # add thread func
-        self._add_thread_func(self._message, True)
+        self._add_thread_func(self._message_func, True)
 
         # store
         self._message = message
         self._interval = interval
 
-    def _message(self):
+    def _message_func(self):
         """Thread function for async processing."""
         # loop until closing
         while not self.closing.is_set():
