@@ -22,7 +22,7 @@ class SkyFlats(Script):
                  flatfield: typing.Union[str, IFlatField], functions: dict,
                  priorities: typing.Union[dict, SkyflatPriorities], min_exptime: float = 0.5, max_exptime: float = 5,
                  timespan: float = 7200, filter_change: float = 30, count: int = 20, combine_binnings: bool = True,
-                 *args, **kwargs):
+                 readout: dict = None, *args, **kwargs):
         """Init a new SkyFlats script.
 
         Args:
@@ -37,6 +37,7 @@ class SkyFlats(Script):
             filter_change: Time required for filter change [s]
             count: Number of flats to schedule
             combine_binnings: Whether different binnings use the same functions.
+            readout: Dictionary with readout times (in sec) per binning (as BxB).
         """
         Script.__init__(self, *args, **kwargs)
 
@@ -55,7 +56,7 @@ class SkyFlats(Script):
         self._scheduler = Scheduler(functions, priorities, self.observer,
                                     min_exptime=min_exptime, max_exptime=max_exptime,
                                     timespan=timespan, filter_change=filter_change, count=count,
-                                    combine_binnings=combine_binnings)
+                                    combine_binnings=combine_binnings, readout=readout)
 
     def can_run(self) -> bool:
         """Whether this config can currently run.
