@@ -1,6 +1,6 @@
 from typing import List
 
-from pyobs.interfaces import ICamera
+from pyobs.utils.enums import ImageType
 from pyobs.utils.time import Time
 
 
@@ -16,13 +16,13 @@ class FrameInfo:
 class Archive:
     def list_options(self, start: Time = None, end: Time = None, night: str = None,
                     site: str = None, telescope: str = None, instrument: str = None,
-                    image_type: ICamera.ImageType = None, binning: str = None, filter_name: str = None,
+                     image_type: ImageType = None, binning: str = None, filter_name: str = None,
                     rlevel: int = None):
         raise NotImplementedError
 
     def list_frames(self, start: Time = None, end: Time = None, night: str = None,
                     site: str = None, telescope: str = None, instrument: str = None,
-                    image_type: ICamera.ImageType = None, binning: str = None, filter_name: str = None,
+                    image_type: ImageType = None, binning: str = None, filter_name: str = None,
                     rlevel: int = None) -> List[FrameInfo]:
         raise NotImplementedError
 
